#include "Base/Axis/Bin.h"
#include "Base/Const/Units.h"
#include "Base/Pixel/SphericalPixel.h"
#include "Base/Vector/Direction.h"
#include "Resample/Element/DiffuseElement.h"
#include "Tests/GTestWrapper/google_test.h"
#include <memory>

namespace {
const Bin1D alpha_bin(0.0 * Units::deg, 1.0 * Units::deg);
const Bin1D phi_bin(-0.5 * Units::deg, 0.5 * Units::deg);
const double wavelength = 42.0;
const double alpha_i = 0.2 * Units::deg;
const double phi_i = 0.0 * Units::deg;
} // namespace

class DiffuseElementTest : public ::testing::Test {
public:
    std::unique_ptr<IPixel> createPixel() const
    {
        return std::make_unique<SphericalPixel>(alpha_bin, phi_bin);
    }

    std::unique_ptr<DiffuseElement> createElement() const
    {
        return std::make_unique<DiffuseElement>(wavelength, alpha_i, phi_i, createPixel(),
                                                Eigen::Matrix2cd{}, Eigen::Matrix2cd{}, false);
    }
};

TEST_F(DiffuseElementTest, basicConstructor)
{
    DiffuseElement ele(wavelength, alpha_i, phi_i, createPixel(), {}, {}, false);
    EXPECT_EQ(ele.wavelength(), wavelength);
    EXPECT_EQ(ele.getAlphaI(), alpha_i);
    EXPECT_EQ(ele.getPhiI(), phi_i);
    EXPECT_EQ(ele.intensity(), 0.0);
    EXPECT_NEAR(ele.getAlphaMean(), 0.5 * Units::deg, 1e-14);
    EXPECT_NEAR(ele.getPhiMean(), 0.0 * Units::deg, 1e-14);
    EXPECT_EQ(ele.getKi(), vecOfLambdaAlphaPhi(wavelength, alpha_i, phi_i));
    EXPECT_EQ(ele.getMeanKf(), createPixel()->getK(0.5, 0.5, wavelength));
    EXPECT_FALSE(ele.isSpecular());
}

TEST_F(DiffuseElementTest, setIntensity)
{
    auto ele = createElement();
    EXPECT_EQ(ele->intensity(), 0.0);
    ele->addIntensity(1.0);
    EXPECT_EQ(ele->intensity(), 1.0);
    ele->setIntensity(42.0);
    EXPECT_EQ(ele->intensity(), 42.0);
}

TEST_F(DiffuseElementTest, copyConstructor)
{
    auto orig = createElement();
    DiffuseElement ele(*orig);
    EXPECT_EQ(orig->wavelength(), ele.wavelength());
    EXPECT_EQ(orig->getAlphaI(), ele.getAlphaI());
    EXPECT_EQ(orig->getPhiI(), ele.getPhiI());
    EXPECT_EQ(orig->getAlphaMean(), ele.getAlphaMean());
    EXPECT_EQ(orig->getPhiMean(), ele.getPhiMean());
    EXPECT_EQ(orig->intensity(), ele.intensity());
    EXPECT_EQ(orig->getKi(), ele.getKi());
    EXPECT_EQ(orig->getMeanKf(), ele.getMeanKf());
    EXPECT_EQ(orig->getQ(0.5, 0.5), ele.getQ(0.5, 0.5));
    EXPECT_EQ(orig->integrationFactor(0.5, 0.5), ele.integrationFactor(0.5, 0.5));
    EXPECT_EQ(orig->solidAngle(), ele.solidAngle());
    EXPECT_EQ(orig->getAlpha(0.5, 0.5), ele.getAlpha(0.5, 0.5));
    EXPECT_EQ(orig->getPhi(0.5, 0.5), ele.getPhi(0.5, 0.5));
    EXPECT_EQ(orig->isSpecular(), ele.isSpecular());
}
